export interface FineTuningParams {
  /**
   * Optional
   * Defaults to 4
   * The number of epochs to train the model for. An epoch refers to one full cycle through the training dataset.
   * 要为其训练模型的纪元数。纪元是指通过训练数据集的一个完整周期。
   */
  n_epochs?: number

  /**
   * Optional
   * Defaults to null
   * The batch size to use for training. The batch size is the number of training examples used to train a single forward and backward pass.
   * By default, the batch size will be dynamically configured to be ~0.2% of the number of examples in the training set, capped at 256 - in general, we've found that larger batch sizes tend to work better for larger datasets.
   * 用于训练的批大小。批量大小是用于训练单个正向和后向传递的训练示例数。
   * 默认情况下，批量大小将动态配置为训练集中样本数的 ~0.2%，上限为 256 - 一般来说，我们发现较大的批量大小往往更适合较大的数据集。
   */
  batch_size?: number

  /**
   * Optional
   * Defaults to null
   * The learning rate multiplier to use for training. The fine-tuning learning rate is the original learning rate used for pretraining multiplied by this value.
   * By default, the learning rate multiplier is the 0.05, 0.1, or 0.2 depending on final batch_size (larger learning rates tend to perform better with larger batch sizes). We recommend experimenting with values in the range 0.02 to 0.2 to see what produces the best results.
   * 用于训练的学习率乘数。微调学习率是用于预训练的原始学习率乘以此值。
   * 默认情况下，学习率乘数为 0.05、0.1 或 0.2，具体取决于最终batch_size（较大的学习率往往在较大的批处理中表现更好）。我们建议尝试 0.02 到 0.2 范围内的值，以查看产生最佳结果的值。
   */
  learning_rate_multiplier?: number

  /**
   * Optional
   * Defaults to 0.01
   * The weight to use for loss on the prompt tokens. This controls how much the model tries to learn to generate the prompt (as compared to the completion which always has a weight of 1.0), and can add a stabilizing effect to training when completions are short.
   * If prompts are extremely long (relative to completions), it may make sense to reduce this weight so as to avoid over-prioritizing learning the prompt.
   * 用于提示令牌损失的权重。这控制了模型尝试学习生成提示的程度（与权重始终为 1.0 的完成相比），并且可以在完成时间较短时为训练添加稳定效果。
   * 如果提示非常长（相对于完成），则减少此权重以避免过度优先学习提示可能是有意义的。
   */
  prompt_loss_weight?: number
}

export interface FineTuningCreateParams extends FineTuningParams {
  /**
   * Required
   * The ID of an uploaded file that contains training data.
   * See upload file for how to upload a file.
   * Your dataset must be formatted as a JSONL file, where each training example is a JSON object with the keys "prompt" and "completion". Additionally, you must upload your file with the purpose fine-tune.
   * See the fine-tuning guide for more details.
   * 包含训练数据的上传文件的 ID。
   * 请参阅上传文件了解如何上传文件。
   * 您的数据集必须格式化为 JSONL 文件，其中每个训练示例都是一个 JSON 对象，带有键“提示”和“完成”。此外，您必须上传具有微调目的的文件。
   * 有关更多详细信息，请参阅微调指南。
   */
  training_file: string

  /**
   * Optional
   * The ID of an uploaded file that contains validation data.
   * If you provide this file, the data is used to generate validation metrics periodically during fine-tuning. These metrics can be viewed in the fine-tuning results file. Your train and validation data should be mutually exclusive.
   * Your dataset must be formatted as a JSONL file, where each validation example is a JSON object with the keys "prompt" and "completion". Additionally, you must upload your file with the purpose fine-tune.
   * See the fine-tuning guide for more details.
   * 包含验证数据的上载文件的 ID。
   * 如果您提供此文件，则数据将用于在微调期间定期生成验证指标。可以在微调结果文件中查看这些指标。训练数据和验证数据应互斥。
   * 数据集必须格式化为 JSONL 文件，其中每个验证示例都是一个 JSON 对象，其键为“提示”和“完成”。此外，您必须上传具有微调目的的文件。
   * 有关更多详细信息，请参阅微调指南。
   */
  validation_file?: string

  /**
   * Optional
   * Defaults to curie
   * The name of the base model to fine-tune. You can select one of "ada", "babbage", "curie", "davinci", or a fine-tuned model created after 2022-04-21. To learn more about these models, see the Models documentation.
   * 要微调的基本模型的名称。您可以选择“ada”、“babbage”、“居里”、“达芬奇”或 2022-04-21 之后创建的微调模型之一。若要了解有关这些模型的详细信息，请参阅模型文档。
   */
  model?: string

  // #region N分类任务
  /**
   * Optional
   * Defaults to false
   * If set, we calculate classification-specific metrics such as accuracy and F-1 score using the validation set at the end of every epoch. These metrics can be viewed in the results file.
   * In order to compute classification metrics, you must provide a validation_file. Additionally, you must specify classification_n_classes for multiclass classification or classification_positive_class for binary classification.
   * 如果设置，我们使用每个纪元结束时的验证集计算特定于分类的指标，例如准确性和 F-1 分数。可以在结果文件中查看这些指标。
   * 为了计算分类指标，您必须提供validation_file。此外，还必须为多类分类指定classification_n_classes或为二元分类指定classification_positive_class。
   * =>
   * 分类任务时设置为true，对话任务不需要设置。这样在训练过程中，每个epoch会打印一次验证集的分类准确率和F1值。
   */
  compute_classification_metrics?: boolean
  /**
   * Optional
   * Defaults to null
   * The number of classes in a classification task.
   * This parameter is required for multiclass classification.
   * 分类任务中的类数。
   * 此参数是多类分类所必需的。
   */
  classification_n_classes?: number
  // #endregion

  // #region 二分类任务配置
  /**
   * Optional
   * Defaults to null
   * The positive class in binary classification.
   * This parameter is needed to generate precision, recall, and F1 metrics when doing binary classification.
   * 二元分类中的正类。
   * 在进行二元分类时，需要此参数来生成精度、召回率和 F1 指标。
   */
  classification_positive_class?: string
  /**
   * Optional
   * Defaults to null
   * If this is provided, we calculate F-beta scores at the specified beta values. The F-beta score is a generalization of F-1 score. This is only used for binary classification.
   * With a beta of 1 (i.e. the F-1 score), precision and recall are given the same weight. A larger beta score puts more weight on recall and less on precision. A smaller beta score puts more weight on precision and less on recall.
   * 如果提供，我们将在指定的 beta 值下计算 F-beta 分数。F-beta 分数是 F-1 分数的概括。这仅用于二元分类。
   * 当 beta 为 1（即 F-1 分数）时，精度和召回率的权重相同。较大的 beta 分数对召回率的重视程度更高，对准确性的重视程度越低。较小的 beta 分数对精度的重视程度更高，对召回率的重视程度越低。
   */
  classification_betas?: number[]
  // #endregion

  /**
   * Optional
   * Defaults to null
   * A string of up to 40 characters that will be added to your fine-tuned model name.
   * For example, a suffix of "custom-model-name" would produce a model name like ada:ft-your-org:custom-model-name-2022-02-15-04-21-04.
   * 最多 40 个字符的字符串，将添加到微调的模型名称中。
   * 例如，“自定义模型名称”的后缀将生成类似 ada：ft-your-org：custom-model-name-2022-02-15-04-21-04 的模型名称。
   */
  suffix?: string
}

export interface OpenResponseBase {
  id: string
  object: string
}

export enum ENUM_FTStatus {
  SUCCEEDED = "succeeded",
  PENDING = "pending",
  FAILED = "failed"
}

export interface BaseFileInfo extends OpenResponseBase {
  bytes: number,
  created_at: number,
  filename: string,
  purpose: string
}
export interface ResultFile extends BaseFileInfo {
  status: string,
  status_details: any | null
}

export interface FineTuningCreateResponse extends OpenResponseBase {
  model?: string
  created_at: number
  events: [
    {
      object: string
      created_at: number,
      level: string,
      message: string
    }
  ],
  fine_tuned_model: string | null
  hyperparams: FineTuningParams,
  organization_id: string,
  result_files: ResultFile[],
  status: ENUM_FTStatus,
  validation_files: BaseFileInfo[],
  training_files: BaseFileInfo[],
  updated_at: string,
}
